/* Program name: listchl.C                                            */
/* History:       January 2002 - created by Ruud van Zundert          */
/* Description:  Example C program to inquire on the channel status   */
/*               attributes of a local queue manager using the        */
/*               MQSeries Administration Interface (MQAI):            */
/*               channel name, status, curmsgs, msgs, shortrts,       */  
/*               indoubt, lstmsgti.                                   */
/*               Caters for both local and remote administration.     */
/*               Displays either all channel statuses, or only those  */
/*               with status 'running' or 'not running'.              */
/* Parameters:   Parm1: the local queue manager name (optional)       */
/*               Parm2: the remote queue manager name (could be same  */
/*                      as parm1)                                     */
/*               Parm3: channel status filter: r (only running),      */
*/                      n (not running  other (all)                   */
/* Includes                              */
#include <stdio.h>
#include <string.h>           
#include <stdlib.h>          
#include <ctype.h>              
#include <cmqc.h>                  /* MQI                             */
#include <cmqcfc.h>                /* PCF                             */
#include <cmqbc.h>                 /* MQAI                            */
/* Function prototypes                                                */
void CheckCallResult(MQCHAR *, MQLONG , MQLONG);
/* Function: main                                                     */
int main(int argc, char *argv[])                 
{
/* -------------- for remote administration only -----------------    */
MQOD     od = {MQOD_DEFAULT};       /* Object Descriptor              */
   MQHOBJ   Hobj;                   /* object handle                  */
   MQLONG   O_options;              /* MQOPEN options                 */
/* ----------------------- MQAI variables --------------------------  */
MQHCONN hConn;                    /* handle to MQ connection          */
MQCHAR qmName[MQ_Q_MGR_NAME_LENGTH+1]=""; /* default or supplied QMgr name */
MQLONG reason;                      /* reason code                    */
MQLONG connReason;                  /* MQCONN reason code             */
MQLONG compCode;                    /* completion code                */
MQHBAG adminBag = MQHB_UNUSABLE_HBAG;  /* admin bag for mqExecute     */
MQHBAG responseBag = MQHB_UNUSABLE_HBAG;/* response bag for mqExecute */ 
MQHBAG chlAttrsBag;                 /* bag containing q attributes    */
MQHBAG errorBag;                    /* bag containing cmd server error*/
MQLONG mqExecuteCC;                 /* mqExecute completion code      */
MQLONG mqExecuteRC;                 /* mqExecute reason code          */
MQLONG actLength;                   /* Actual length of parameter     */
MQLONG i;                           /* loop counter                   */
MQLONG numberOfBags;                /* number of bags in response bag */
MQLONG qHandle;                     /* either MQHO_NONE or real handle*/
MQCHAR staQual[2];                  /* chl status  qualifier/filter   */
MQLONG ChlType,CurMsgs, Msgs, ShortRts, InDoubt, ChStat;
MQCHAR ChlName[MQ_CHANNEL_NAME_LENGTH+1]; /* channel name             */
MQCHAR LastTime[MQ_CHANNEL_TIME_LENGTH+1]; /* last message time       */
MQCHAR wChlStat[13], wChlType[9];   /* channel status and type        */
printf("listchl - C program which uses the MQAI to display channel status\n\n");
printf("%-20s %12s %8s %8s %8s %8s %12s %8s\n\n", "Channel Name", "Chl Type",
     "CurMsgs", "Msgs", "ShortRts", "InDoubt", "Chl Status", "LastMsg");
/* ------ Connect to the queue manager - Default or supplied ---- --- */
NOTE 1: if (argc > 1)               /* Use qmgr if supplied           */
      strncpy(qmName, argv[1], (size_t)MQ_Q_MGR_NAME_LENGTH);
   MQCONN(qmName, &hConn, &compCode, &connReason);    
/* ----- Report the reason and stop if the connection failed. ---- -- */
   if (compCode == MQCC_FAILED)
   {
      CheckCallResult("Queue Manager connection", compCode, connReason);
      exit( (int)connReason);
   }
/* if a remote qmgr was supplied, set up the object descriptor, and open it*/
NOTE 2: qHandle = MQHO_NONE;        /* Default: use local cmd queue   */
   if (argc > 2)                    /* if rmt qmgr given, store it in */
   {                                /* the MQMD for remote admin      */
       strncpy(od.ObjectQMgrName, argv[2], (size_t)MQ_Q_MGR_NAME_LENGTH);
       strncpy(od.ObjectName, "SYSTEM.ADMIN.COMMAND.QUEUE", (size_t)MQ_Q_NAME_LENGTH);
       O_options = MQOO_OUTPUT      /* open queue for output          */
        + MQOO_FAIL_IF_QUIESCING;   /* but not if MQM stopping        */
       MQOPEN(hConn,                /* connection handle              */
       &od,                         /* object descriptor for queue    */
               O_options,           /* open options                   */
               &Hobj,               /* object handle                  */
               &compCode,           /* MQOPEN completion code         */
               &reason);            /* reason code                    */
       qHandle = Hobj;              /* store for mqExecute use        */
if (reason != MQRC_NONE)
               printf ("MQOPEN ended with reason code %ld\n", reason);
       if (compCode == MQCC_FAILED)
               printf("unable to open queue for output\n");
   }
NOTE 3: memset (staQual, '\0', sizeof(staQual) ); /* Initialize status qualifier  */
   if ( argc > 3 )                   /* if parm3 supplied, use it     */
       strncpy(staQual, argv[3] , sizeof(staQual) );
/* ------ Create an admin request bag for the mqExecute call -------  */
NOTE 4: mqCreateBag(MQCBO_ADMIN_BAG, &adminBag, &compCode, &reason);
   CheckCallResult("Create admin bag", compCode, reason);
/* -------Create an admin response bag for the mqExecute call ------- */
NOTE 5: mqCreateBag(MQCBO_ADMIN_BAG, &responseBag, &compCode, &reason);
   CheckCallResult("Create response bag", compCode, reason);
/* -------Put the generic channel name into the admin bag ----------- */
NOTE 6: mqAddString(adminBag, MQCACH_CHANNEL_NAME, MQBL_NULL_TERMINATED, "*",
               &compCode, &reason);
   CheckCallResult("Add clusqmgr name", compCode, reason);
/* ------- Add an inquiry for various channel attributes ------------ */
NOTE 7: mqAddInquiry(adminBag, MQIACH_CHANNEL_TYPE, &compCode, &reason);
   CheckCallResult("Add chl type", compCode, reason);
mqAddInquiry(adminBag, MQIACH_CURRENT_MSGS, &compCode, &reason);
   CheckCallResult("Add chl curmsgs", compCode, reason);
mqAddInquiry(adminBag, MQIACH_MSGS, &compCode, &reason);
   CheckCallResult("Add chl msgs", compCode, reason);
mqAddInquiry(adminBag, MQIACH_SHORT_RETRIES_LEFT, &compCode, &reason);
   CheckCallResult("Add chl short retries", compCode, reason);
mqAddInquiry(adminBag, MQIACH_INDOUBT_STATUS, &compCode, &reason);
   CheckCallResult("Add chl indoubt", compCode, reason);
mqAddInquiry(adminBag, MQIACH_CHANNEL_STATUS, &compCode, &reason);
   CheckCallResult("Add chl status", compCode, reason);
mqAddInquiry(adminBag, MQCACH_LAST_MSG_TIME, &compCode, &reason);
   CheckCallResult("Add chl status", compCode, reason);
/* Send the command to find all local queue names and our attributes. */
/* The mqExecute call creates the PCF structure required, sends it to */
/* the command server, and receives the reply from the command server */
/* into the response bag. The attributes are contained in system bags */
/* embedded in the response bag, one set of attributes per bag.       */
/* By default, mqExecute waits for up to 30 seconds for a response.   */
NOTE 8: mqExecute(hConn,       /* MQ connection handle                */
          MQCMD_INQUIRE_CHANNEL_STATUS, /* Command to be executed     */
          MQHB_NONE,                    /* No options bag             */
          adminBag,          /* Handle to bag containing commands     */
          responseBag,       /* Handle to bag to receive the response */
          qHandle,           /* Put msg on SYSTEM.ADMIN.COMMAND.QUEUE */  
          MQHO_NONE,         /* Create a dynamic q for the response   */ 
          &compCode,         /* Completion code from the mqexecute    */ 
          &reason);          /* Reason code from mqexecute call       */
   
NOTE 9: /* -- Check the command server is started.Only true for local qmgr admin --*/
   if (reason == MQRC_CMD_SERVER_NOT_AVAILABLE)
   {
     printf("Please start the command server: <strmqcsv QMgrName>\n");
     MQDISC(&hConn, &compCode, &reason);
     CheckCallResult("Disconnect from Queue Manager", compCode, reason);         
     exit(98);
   }  
/* - Check the result from mqExecute call. If successful find all the */
/* - req'd attributes of all local queues. If failed find the error.  */
if ( compCode == MQCC_OK )                 /* Successful mqExecute    */
   {
/* Count number of system bags embedded in the response bag from the  */
/* mqExecute call. Attributes for each queue are in a separate bag.   */
NOTE 10:  mqCountItems(responseBag, MQHA_BAG_HANDLE, &numberOfBags, &compCode, &reason);
     CheckCallResult("Count number of bag handles", compCode, reason); 
for ( i=0; i<numberOfBags; i++)
     {   
/* Get the next system bag handle out of the mqExecute response bag.  */
/* This bag contains the queue attributes                             */ 
NOTE 11: mqInquireBag(responseBag, MQHA_BAG_HANDLE, i, &chlAttrsBag, &compCode, &reason);
         CheckCallResult("Get the result bag handle", compCode, reason);
/* ------ Get the various channel attributes out of the bag ------    */
NOTE 12: mqInquireString(chlAttrsBag, MQCACH_CHANNEL_NAME, 0, MQ_CHANNEL_NAME_LENGTH,
   ChlName,  &actLength, NULL, &compCode, &reason);
CheckCallResult("Get channel name", compCode, reason); 
mqInquireInteger(chlAttrsBag, MQIACH_CHANNEL_TYPE, MQIND_NONE, &ChlType, 
   &compCode, &reason);
CheckCallResult("Get chl type", compCode, reason);
mqInquireInteger(chlAttrsBag, MQIACH_CURRENT_MSGS, MQIND_NONE, &CurMsgs, 
   &compCode, &reason);
CheckCallResult("Get chl curmsgs", compCode, reason);
mqInquireInteger(chlAttrsBag, MQIACH_MSGS, MQIND_NONE, &Msgs, 
   &compCode, &reason);
CheckCallResult("Get chl msgs", compCode, reason);
mqInquireInteger(chlAttrsBag, MQIACH_SHORT_RETRIES_LEFT, MQIND_NONE, 


CheckCallResult("Get chl shorrts", compCode, reason);
mqInquireInteger(chlAttrsBag, MQIACH_INDOUBT_STATUS, MQIND_NONE,  


CheckCallResult("Get chl indoubt", compCode, reason);
mqInquireInteger(chlAttrsBag, MQIACH_CHANNEL_STATUS, MQIND_NONE,


CheckCallResult("Get chl status", compCode, reason);
mqInquireString(chlAttrsBag, MQCACH_LAST_MSG_TIME, 0, 


CheckCallResult("Get last msg time", compCode, reason);
/* ----- Use mqTrim to prepare the queue name for printing. --------  */
/* ----- Print the result dependant upon the filtering required  ---- */
NOTE 13:    if ( ((strncmp (staQual, "r", 1) == 0) && (ChStat == 3) ) ||
 ((strncmp (staQual, "n", 1) == 0) && (ChStat != 3) ) ||
 ((strncmp (staQual, "r", 1) != 0) && (strncmp (staQual, "n", 1) != 0)))
        {
NOTE 14: switch ( ChlType )
        {
               case 1: { strncpy(wChlType,"Sender      ", 12); break; }
               case 2: { strncpy(wChlType,"Server      ", 12); break; }
               case 3: { strncpy(wChlType,"Receiver    ", 12); break; }
               case 4: { strncpy(wChlType,"Requestor   ", 12); break; }
               case 5: { strncpy(wChlType,"All         ", 12); break; }
               case 6: { strncpy(wChlType,"ClientConn  ", 12); break; }
               case 7: { strncpy(wChlType,"ServerConn  ", 12); break; }
               case 8: { strncpy(wChlType,"Cluster Rcvr", 12); break; }
               case 9: { strncpy(wChlType,"Cluster Sdr ", 12); break; }
               default: {strncpy(wChlType,"Unknown     ", 12); }
        };
        switch ( ChStat )
        {
               case 0: { strncpy(wChlStat,"Inactive    ", 12); break; }
               case 1: { strncpy(wChlStat,"Binding     ", 12); break; }
               case 2: { strncpy(wChlStat,"Starting    ", 12); break; }
               case 3: { strncpy(wChlStat,"Running     ", 12); break; }
               case 4: { strncpy(wChlStat,"Stopping    ", 12); break; }
               case 5: { strncpy(wChlStat,"Retrying    ", 12); break; }
               case 6: { strncpy(wChlStat,"Stopped     ", 12); break; }
               case 7: { strncpy(wChlStat,"Requesting  ", 12); break; }
               case 8: { strncpy(wChlStat,"Paused      ", 12); break; }
               case 13: {strncpy(wChlStat,"Initialising", 12); break; }
               default: {strncpy(wChlStat,"Unknown     ", 12); }
         };
mqTrim(MQ_CHANNEL_NAME_LENGTH, ChlName, ChlName, &compCode, &reason);
mqTrim(12, wChlType, wChlType, &compCode, &reason);
mqTrim(MQ_CHANNEL_TIME_LENGTH, LastTime, LastTime, &compCode, &reason);
mqTrim(12, wChlStat, wChlStat, &compCode, &reason);
printf("%-20s %12s %8ld %8ld %8ld %8ld %12s %8s\n", ChlName, wChlType, 
   CurMsgs, Msgs, ShortRts, InDoubt, wChlStat, LastTime);
        }
     } 
   }
NOTE 15: else                                 /* Failed mqExecute     */
   {
     printf("Call to get queue attr failed: Completion Code = %ld : Reason = %ld\n", 
compCode, reason);
/* If the command fails to get the system bag handle out of the       */
/* mqexecute response bag. This bag contains the reason from the      */
/* command server why the command failed.                             */ 
     if (reason == MQRCCF_COMMAND_FAILED)
     {
mqInquireBag(responseBag, MQHA_BAG_HANDLE, 0, &errorBag, &compCode,  


CheckCallResult("Get the result bag handle", compCode, reason);
/* Get the completion code and reason code, returned by the command   */
/* server, from the embedded error bag.                               */
mqInquireInteger(errorBag, MQIASY_COMP_CODE, MQIND_NONE, &mqExecuteCC, 
   &compCode, &reason );
CheckCallResult("Get the completion code from the result bag", compCode,
 reason); 
mqInquireInteger(errorBag, MQIASY_REASON, MQIND_NONE, &mqExecuteRC,
   &compCode, &reason);
CheckCallResult("Get the reason code from the result bag", compCode,  


printf("Error from the cmd server: Completion Code = %ld : Reason = %ld\n", mqExecuteCC, mqExecuteRC);       
     }
   }  
/* --------- Delete the admin bag if successfully created. ---------  */
NOTE 16: if (adminBag != MQHB_UNUSABLE_HBAG)
   {
      mqDeleteBag(&adminBag, &compCode, &reason);
      CheckCallResult("Delete the admin bag", compCode, reason);
   }
/* -------- Delete the response bag if successfully created. -------- */
NOTE 17: if (responseBag != MQHB_UNUSABLE_HBAG)
   {
      mqDeleteBag(&responseBag, &compCode, &reason);
      CheckCallResult("Delete the response bag", compCode, reason);
   }
/* --- Disconnect from the queue manager if not already connected --- */
NOTE 18: if (connReason != MQRC_ALREADY_CONNECTED)
   {
      MQDISC(&hConn, &compCode, &reason);
      CheckCallResult("Disconnect from Queue Manager", compCode, reason);         
   } 
   return 0;                               
}
/* Logic: Display the description of the call, completion code and    */
/* reason code if the completion code is not successful               */
NOTE 19: void  CheckCallResult(char *callText, MQLONG cc, MQLONG rc)
{
   if (cc != MQCC_OK)
     printf("%s failed: Completion Code = %ld : Reason = %ld\n",
       callText, cc, rc);
}

/***********************
C:\>listchl QM1 JAV1
listchl - example C program which uses the MQAI to display channel status
Channel    Chl Type  CurMsgs  Msgs ShortRts InDoubt  Chl       LastMsg
Name                                                 Status
HUB2.JAV3  Sender      0      0      0       0       Retrying
HUB2.JAV2  Sender      0      0      9       0       Retrying
HUB2.HUB1  Sender      0      1     10       0       Running   15.35.06
HUB1.JAV1  Receiver    0      0     10       0       Running


C:\>listchl QM1 JAV1 r
listchl - example C program which uses the MQAI to display channel status

Channel    Chl Type  CurMsgs  Msgs ShortRts  InDoubt  Chl       LastMsg
Name                                                  Status  
HUB2.HUB1  Sender     0       5       10        0     Running   15.40.05
HUB1.JAV1  Receiver   0       1       10        0     Running   15.35.06


C:\>listchl QM1 JAV1
listchl - example C program which uses the MQAI to display channel status
Channel    Chl Type  CurMsgs  Msgs ShortRts InDoubt  Chl       LastMsg
Name                                                 Status
HUB2.JAV3  Sender      0         0       0     0     Retrying
HUB2.JAV2  Sender      0         0       0     0     Retrying
HUB2.HUB1  Sender      0        13      10     0     Running   16.52.21
HUB1.JAV1  Receiver    0         3      10     0     Running   16.52.04
Get chl curmsgs failed: Completion Code = 2 : Reason = 2309
Get chl shorrts failed: Completion Code = 2 : Reason = 2309
Get chl indoubt failed: Completion Code = 2 : Reason = 2309  
SYSTEM.ADMIN.SVRCONN   0         18     10     0     Running   16.52.14
   ServerConn
***********************/